
#include <pspaudio.h>
#include "pspcommon.h"
#include "shared.h"
#include "sound.h"
#include "SMS.h"
#include "pspsound.h"

typedef struct {
        short l, r;
} sample_t;

//#define MAXSAMPLES 4096
//4096 marche aussi mais augmente les risques de clics
#define MAXSAMPLES 8192
sample_t audioOut[2048], audioOutTemp[MAXSAMPLES];
int nbreSamples=0, ptrSample=0;
int sndThid = 0;
static int SoundThread();
int soundPause = 1, soundInited = 0;
s64 total1=0, total2=0;

void SoundInit(void)
{
	if(soundInited)
		SoundTerm();
	if (!soundInited)		{
		if(snd.sample_rate)
		{
			memset(audioOut, 0, 2048*sizeof(sample_t));

			sndThid = sceKernelCreateThread("SoundThread", SoundThread, 0x18, 0xFA0, 0, 0); 

			if(sndThid<0)
				return;

			sceKernelStartThread(sndThid,0,0);  
			soundInited = 1;
		}
	}
}

void SoundTerm(void)
{
	if(soundInited)
	{
		sceKernelTerminateDeleteThread(sndThid);
		sndThid = 0;
		soundInited = 0;
	}
}

void SoundUpdate(void)
{
	if(menuConfig.sound.enabled)
	{
		int i;
		if (menuConfig.sound.volume)		{
			for(i = 0; i < snd.sample_count; ++i) 
			{
				audioOutTemp[nbreSamples].l = (snd.output[0][i]<<menuConfig.sound.volume);
				audioOutTemp[nbreSamples].r = (snd.output[1][i]<<menuConfig.sound.volume);
				nbreSamples = (nbreSamples + 1) & (MAXSAMPLES - 1);
			}
		}
		else	{
			for(i = 0; i < snd.sample_count; ++i) 
			{
				audioOutTemp[nbreSamples].l = (snd.output[0][i]);
				audioOutTemp[nbreSamples].r = (snd.output[1][i]);
				nbreSamples = (nbreSamples + 1) & (MAXSAMPLES - 1);
			}
		}

		total1 += snd.sample_count;
		soundPause = 0;
	}
}

void SoundPause(void)
{
	soundPause = 1;
	
	//sceKernelSuspendThread(sndThid);
}

void SoundResume(void)
{
	soundPause = 0;

	//sceKernelResumeThread(sndThid);
}

int AudioGet(int sample_count, int realSampleCount)
{
	int i;
	/*
		En cas normal, le chip son s'attend  avoir l'quivalent de 44100 samples par seconde.
		Dans notre cas, 44100 est divis en 60 frames par seconde, mais en ralit la synchro LCD s'effectue seulement
		 59.xx fps, rsultat on prend priodiquement du retard. Lorsque ce retard est trop important, on saute
		simplement une sample.
	*/
	if (total2 + realSampleCount > total1)
		return;
	/*
		Au cas o le son aurait de l'avance sur nous, on doit le rattraper.
		Problme: si on utilise le frameskip, le son peut prendre une immense avance sur nous car x frames sont gnres
		rapidement avant d'tre effectivement joues, on doit donc tre sr que ce n'est pas le cas avec frameReady
	*/
	if (frameReady)			{
		while (total2 + sample_count < total1 - sample_count * 4)		{
			total2 += sample_count;
			ptrSample = (ptrSample + sample_count) & (MAXSAMPLES - 1);
		}
		frameReady = 0;
	}
	if (menuConfig.sound.sampleRate == 44100)			{
		if (ptrSample + sample_count < MAXSAMPLES)		{
			memcpy(audioOut, audioOutTemp + ptrSample, sample_count * sizeof(*audioOut));
			ptrSample += sample_count;
		}
		else		{
			int part1 = MAXSAMPLES - ptrSample;
			memcpy(audioOut, audioOutTemp + ptrSample, part1 * sizeof(*audioOut));
			memcpy(audioOut + part1, audioOutTemp, (sample_count - part1) * sizeof(*audioOut));
			ptrSample = (ptrSample + sample_count) & (MAXSAMPLES - 1);
		}
	}
	else if (menuConfig.sound.sampleRate == 22050)			{
		for(i = 0; i < sample_count; ++i)
		{
			audioOut[i].l = audioOutTemp[ptrSample].l;
			audioOut[i].r = audioOutTemp[ptrSample].r;
			++i;
			audioOut[i].l = audioOutTemp[ptrSample].l;
			audioOut[i].r = audioOutTemp[ptrSample].r;
			ptrSample = (ptrSample + 1) & (MAXSAMPLES - 1);
		}
	}
	else if (menuConfig.sound.sampleRate == 11025)			{
		for(i = 0; i < sample_count; ++i)
		{
			audioOut[i].l = audioOutTemp[ptrSample].l;
			audioOut[i].r = audioOutTemp[ptrSample].r;
			++i;
			audioOut[i+1].l = audioOutTemp[ptrSample].l;
			audioOut[i+1].r = audioOutTemp[ptrSample].r;
			++i;
			audioOut[i+2].l = audioOutTemp[ptrSample].l;
			audioOut[i+2].r = audioOutTemp[ptrSample].r;
			++i;
			audioOut[i+3].l = audioOutTemp[ptrSample].l;
			audioOut[i+3].r = audioOutTemp[ptrSample].r;
			ptrSample = (ptrSample + 1) & (MAXSAMPLES - 1);
		}
	}
	total2 += realSampleCount;
}


static int SoundThread()
{
	int err=0;
	int sample_count = PSP_AUDIO_SAMPLE_ALIGN(snd.sample_rate/snd.fps);
	int chan = sceAudioChReserve(PSP_AUDIO_NEXT_CHANNEL, sample_count, PSP_AUDIO_FORMAT_STEREO);
	int realSampleCount;
	realSampleCount = sample_count / (44100 / menuConfig.sound.sampleRate);

	if(chan > 0)
	{
		for(;;)
		{
			if (soundPause)
			{
				memset(audioOut, 0, 2048*sizeof(sample_t));
			}

			AudioGet(sample_count, realSampleCount);
			sceAudioOutputPannedBlocking(chan, PSP_AUDIO_VOLUME_MAX, PSP_AUDIO_VOLUME_MAX, (void*)audioOut);
		}
	}
}
