
#include <pspkernel.h>
#include <pspmodulemgr.h>
#include <pspdisplay.h>
#include <pspdebug.h>
#include <pspthreadman.h>
#include <pspctrl.h>
#include <pspsdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "shared.h"
#include "SMS.h"
#include "system.h"
#include "pspcommon.h"

int systemInit = 0;
int frameReady = 0;
unsigned char pixel_values[256];

int MenuPlusAction(int action, void* param)			{
	switch (action)			{
		case MA_LOADROM:
			strcpy(menuConfig.file.filename, (char*)param);
			if (systemInit)
			{
				system_poweroff();
			}

			if(load_rom(menuConfig.file.filename) == 0) 
			{
				return 0;
			}

			if(systemInit)
			{
				sms_init();
				render_init();
				system_poweron();
			}
			menuPlusTerminated = 1;
			break;
	}
	return 1;
}

//moment: -1 = avant, 0 = toujours, +1 = aprs
void MenuOptionsConfigure(int moment)		{
	static MENUPARAMS *m = NULL;
	if (moment == -1)		{
		if (!m)
			m = malloc(sizeof(*m));
		if (!m)
			oslFatalError("MenuOptionsConfigure#1: m == NULL");
		memcpy(m, &menuConfig, sizeof(*m));
	}
	else	{
		if (!m)
			oslFatalError("MenuOptionsConfigure#2: m == NULL");
		if (moment == 0 ||
			m->video.brightnessMode != menuConfig.video.brightnessMode ||
			m->video.gamma != menuConfig.video.gamma)			{
			int i;
			for (i=0;i<256;i++)			{
				if (menuConfig.video.brightnessMode == 0)
					//Normal
					pixel_values[i] = i >> 3;
				else if (menuConfig.video.brightnessMode == 1)
					//Bright & Fast
					pixel_values[i] = (((i * 232) >> 8) + 24) >> 3;
				else
					//Gamma
					pixel_values[i] = ((unsigned char)(255.0f * pow((i / 255.0f), 1.0f/(((float)menuConfig.video.gamma) / 100.f)))) >> 3;
			}
		}
		if (moment == 0 ||
			m->video.renderDepth != menuConfig.video.renderDepth)		{
			bitmap.depth  = menuConfig.video.renderDepth;
			bitmap.pitch  = 256 * menuConfig.video.renderDepth / 8;
		}
		//Le son a chang?
		if (moment == 0 ||
			m->sound.enabled != menuConfig.sound.enabled ||
			m->sound.sampleRate != menuConfig.sound.sampleRate ||
			m->sound.stereo != menuConfig.sound.stereo ||
			m->video.country != menuConfig.video.country)			{
			//Son
			if(menuConfig.sound.enabled)
			{
				snd.sample_rate    = menuConfig.sound.sampleRate;
				snd.fps            = (menuConfig.video.country == 1) ? 50 : 60;
				snd.psg_clock      = 3579545;
				snd.mixer_callback = NULL;
				sms.use_fm         = 0;
			}
			else
				snd.sample_rate    = 0;

			//Son
			if(menuConfig.sound.enabled)
				SoundInit();
			else
				SoundTerm();

			sound_init();
		}
		if (moment == 0 || m->misc.pspClock != menuConfig.misc.pspClock)		{
			if (menuConfig.misc.pspClock >= 100 && menuConfig.misc.pspClock <= 333)
				scePowerSetClockFrequency (menuConfig.misc.pspClock, menuConfig.misc.pspClock, menuConfig.misc.pspClock / 2);
		}
		free(m);
		m = NULL;
	}
}

void SmsInit(void)
{

//	sceDisplayWaitVblankStart();
//	sceGuSwapBuffers();

	systemInit = 1;
}

void SmsTerm(void)
{
	if(systemInit)
	{
		// Close any network resources, the check is done in the network modules
//		adhocTerm();

		SoundTerm();
		ControlsTerm();
		VideoGuTerm();

		system_poweroff();
		system_shutdown();
	}

	systemInit = 0;
}

extern volatile int osl_vblCount;
int gblFramerate = 0, gblVirtualFramerate = 0;

void SmsEmulate()
{
	u32 nFrame, nDrawnFrames, skip=0;
	int nbRenderedFramesPerSec, nbVirtualFramesPerSec, lastVCount, vsyncAdd;

	memset(&bitmap, 0, sizeof(bitmap_t));
	bitmap.width  = 256;
	bitmap.height = 192;
	bitmap.depth  = menuConfig.video.renderDepth;
	bitmap.pitch  = 256 * menuConfig.video.renderDepth / 8;
	ControlsInit();
	VideoGuInit();

	menuConfig.file.filename[0] = '\0';

	sms.save = 1;

	// Display the menu to load up the ROM or connect to an adhoc server
//	displayMenu();
	menuIsInGame = 0;
	menuPlusShowMenu();
//	MenuPlusAction(MA_LOADROM, "ms0:/PSP/GAME/SMSPlus1.2/roms/Sonic Blast.sms");

	//Bidouille pour dtecter les choses qui ont chang
	MenuOptionsConfigure(-1);
	MenuOptionsConfigure(0);

	//Systme
	system_init();
	system_poweron();
	VideoGuScreenClear();

	//Systme
	systemInit = 1;
	nFrame = nDrawnFrames = 0;
	menuConfig.video.turbo = menuConfig.video.pause = 0;
	nbRenderedFramesPerSec = 0;
	gblFramerate = gblVirtualFramerate = 0;
	menuUpdateRender = 0;

	while(1)
	{
		if (menuUpdateRender)
			nDrawnFrames = 0, menuUpdateRender = 0;
		ControlsUpdate();
		if (osl_quit)
			break;

		//Framerate
		if (menuConfig.video.cpuTime == 2)			{
			if (!skip)
				nbRenderedFramesPerSec++;
			nbVirtualFramesPerSec++;
			if (osl_vblCount / 60 > lastVCount / 60)			{
				gblFramerate = nbRenderedFramesPerSec;
				gblVirtualFramerate = nbVirtualFramesPerSec;
				nbRenderedFramesPerSec = 0;
				nbVirtualFramesPerSec = 0;
			}
			lastVCount = osl_vblCount;
		}

		if (!menuConfig.video.pause)			{
			system_frame(skip);
			SoundUpdate();
		}

		if (!skip)			{
			VideoGuUpdate(nDrawnFrames, menuConfig.video.render);
			frameReady = 1;
			nDrawnFrames++;
		}

		vsyncAdd = 0;

redo:
		if (menuConfig.video.turbo)			{
			if (!skip)
				oslSwapBuffers();
			skip = nFrame % (menuConfig.video.turboSkip + 1);
		}
		else	{
			int vsync = 4 + vsyncAdd;
			if (menuConfig.video.vsync == 2)
				vsync |= 1;
			else if (menuConfig.video.vsync == 1)
				vsync |= 8;
			skip = oslSyncFrameEx(menuConfig.video.frameskip, menuConfig.video.frameskipMax, vsync);
		}
		nFrame++;
		//Pal
		if (menuConfig.video.country == 1 && nFrame % 6 == 0)		{
			if (!skip)
				vsyncAdd = 16;
			goto redo;
		}
		menuIsInGame = 1;
	}

	SmsTerm();
}

/* Save or load SRAM */
void system_manage_sram(uint8 *sram, int slot, int mode)
{
    char name[MAX_PATH];
    FILE *fd;

	if(menuConfig.file.wifiEnabled)
	{
		memset(sram, 0x00, 0x8000);
		return;
	}

	if(menuConfig.file.filename[0] == '\0')
		return;

    strcpy(name, menuConfig.file.filename);
    strcpy(strrchr(name, '.'), ".sram");

    switch(mode)
    {
        case SRAM_SAVE:
//            if(sms.save)
//            {
				if (BatteryWarning("Your battery is low!\nDo you want to save the SRAM contents?\n(This might corrupt your Memory Stick if your PSP stops during this operation.)"))			{
					fd = fopen(name, "wb");
					if(fd)
					{
						fwrite(sram, 0x8000, 1, fd);
						fclose(fd);
					}
				}
//            }
            break;

        case SRAM_LOAD:
            fd = fopen(name, "rb");
            if(fd)
            {
                sms.save = 1;
                fread(sram, 0x8000, 1, fd);
                fclose(fd);
            }
            else
            {
                /* No SRAM file, so initialize memory */
                memset(sram, 0x00, 0x8000);
            }
            break;
    }
}

