
#include <pspdisplay.h>
#include <pspgu.h>
#include "shared.h"
#include "VideoGu.h"
#include "SMS.h"
#include "pspcommon.h"
#include "logo/logo.h"
#include "logo/neoflash.h"

//uint32 pspGuList [262144] __attribute__((aligned(16)));
screen_t Screen __attribute__((aligned(64)));
int videoInited = 0;

struct Vertex
{
  unsigned short u, v;
  short x, y, z;
};

void pspScaleImage(char *pixels, int srcX0, int srcY0, int srcX1, int srcY1, 
				   int srcPitch, int dstX0, int dstY0, int dstX1, int dstY1);

void VideoGuInit(void)
{
/*	sceGuInit();

	sceGuStart(0, pspGuList);
	sceGuDrawBufferList(GU_PSM_5551, (void *)0, 512);
	sceGuDispBuffer(480, 272, (void *)0x88000, 512);
	sceGuDepthBuffer((void *)0x110000, 512);
	sceGuOffset(2048 - (480/2), 2048 - (272/2));
	sceGuViewport(2048, 2048, 480, 272);
	sceGuDepthRange(0xc350, 0x2710);
	sceGuScissor(0, 0, 480, 272);
	sceGuEnable(GU_SCISSOR_TEST);
	sceGuFrontFace(GU_CW);
	sceGuEnable(GU_TEXTURE_2D);
	sceGuClear(GU_COLOR_BUFFER_BIT | GU_DEPTH_BUFFER_BIT);
	sceGuFinish();

	sceGuSync(0, 0);
	sceDisplayWaitVblankStart();
	sceGuDisplay(1);*/

	if (!videoInited)		{
		oslInitGfx(OSL_PF_5650, 1);
		oslInitConsole();
		oslStartDrawing();
		oslEndDrawing();
		ShowNeoflashLogo();
		logoAffiche();
		VideoGuScreenClear();
		bitmap.data   = (unsigned char*)Screen.scrBuffer;
		videoInited = 1;
	}
}

void VideoGuTerm(void)
{
	sceGuTerm();
}

void DisplayBenchmark()			{
	if (menuConfig.video.cpuTime == 1)		{
		int ms6;
		ms6 = oslMeanBenchmarkTestEx(OSL_BENCH_GET, 6);
		oslPrintf_xy(0,0, "%i.%i%%", (ms6 * 6) /1000, ((ms6 * 6) % 1000) / 100);
	}
	else if (menuConfig.video.cpuTime == 2)		{
		oslPrintf_xy(0,0, "%i fps (%i%%)", gblFramerate, 100 * gblVirtualFramerate / snd.fps);
	}
}

void VideoGuUpdate_Core(int fullScreen)		{
	int viewPortX = bitmap.viewport.x, viewPortW = bitmap.viewport.w;
	if (!IS_GG && !menuConfig.video.leftBar)		{
		bitmap.viewport.x = max(bitmap.viewport.x, 8);
		bitmap.viewport.w -= bitmap.viewport.x - viewPortX;
	}
	//1x
	if(fullScreen == 0)
	{
		int x = (480 - bitmap.viewport.w) / 2, y = (272 - bitmap.viewport.h) / 2;
		pspScaleImage((char *)&Screen.scrBuffer, 
						bitmap.viewport.x,
						bitmap.viewport.y,
						bitmap.viewport.x + bitmap.viewport.w,
						bitmap.viewport.y + bitmap.viewport.h,
						256,
						x, y, x + bitmap.viewport.w, y + bitmap.viewport.h);
	} 
	//2x
	else if (fullScreen == 1)
	{
		int x = 480 / 2 - bitmap.viewport.w, y = 272 / 2 - bitmap.viewport.h;
		pspScaleImage((char *)&Screen.scrBuffer, 
						bitmap.viewport.x,
						bitmap.viewport.y,
						bitmap.viewport.x + bitmap.viewport.w,
						bitmap.viewport.y + bitmap.viewport.h,
						256,
						x, y, x + bitmap.viewport.w * 2, y + bitmap.viewport.h * 2);
	}
	//Fit
	else if (fullScreen == 2)
	{
		int scale = min((4096 * 480) / bitmap.viewport.w, (4096 * 272) / bitmap.viewport.h);
		int x = 480 / 2 - (((bitmap.viewport.w * scale) >> 12) / 2),
			y = 272 / 2 - (((bitmap.viewport.h * scale) >> 12) / 2);
		pspScaleImage((char *)&Screen.scrBuffer, 
						bitmap.viewport.x,
						bitmap.viewport.y,
						bitmap.viewport.x + bitmap.viewport.w,
						bitmap.viewport.y + bitmap.viewport.h,
						256,
						x, y, x + ((bitmap.viewport.w * scale) >> 12), y + ((bitmap.viewport.h * scale) >> 12));
	}
	//Wide
	else if (fullScreen == 3)
	{
		int scaleY = min((4096 * 480) / bitmap.viewport.w, (4096 * 272) / bitmap.viewport.h);
		int scaleX = (scaleY + (4096 * 480) / bitmap.viewport.w) / 2;
		int x = 480 / 2 - (((bitmap.viewport.w * scaleX) >> 12) / 2),
			y = 272 / 2 - (((bitmap.viewport.h * scaleY) >> 12) / 2);
		pspScaleImage((char *)&Screen.scrBuffer, 
						bitmap.viewport.x,
						bitmap.viewport.y,
						bitmap.viewport.x + bitmap.viewport.w,
						bitmap.viewport.y + bitmap.viewport.h,
						256,
						x, y, x + ((bitmap.viewport.w * scaleX) >> 12), y + ((bitmap.viewport.h * scaleY) >> 12));
	}
	//1.5x
	else if (fullScreen == 5)
	{
		int x = 480 / 2 - (3 * bitmap.viewport.w) / 4, y = 272 / 2 - (3 * bitmap.viewport.h) / 4;
		pspScaleImage((char *)&Screen.scrBuffer, 
						bitmap.viewport.x,
						bitmap.viewport.y,
						bitmap.viewport.x + bitmap.viewport.w,
						bitmap.viewport.y + bitmap.viewport.h,
						256,
						x, y, x + (bitmap.viewport.w * 3) / 2, y + (bitmap.viewport.h * 3) / 2);
	}
	//Full
	else
	{
		pspScaleImage((char *)&Screen.scrBuffer, 
						bitmap.viewport.x,
						bitmap.viewport.y,
						bitmap.viewport.x + bitmap.viewport.w,
						bitmap.viewport.y + bitmap.viewport.h,
						256,
						0, 0, 480, 272);
	}
	bitmap.viewport.x = viewPortX;
	bitmap.viewport.w = viewPortW;
}

void VideoGuUpdate(int nFrame, int fullScreen)
{
	oslStartDrawing();
	//Pour le texte
	oslSetTextColor(RGB(255,255,255));
	oslSetBkColor(RGBA(0,0,0,0x80));
	if (menuStatusDrawn > 0 || nFrame < 3)
		oslClearScreen(0);

	VideoGuUpdate_Core(fullScreen);

	if (menuConfig.video.cpuTime)
		DisplayBenchmark();
	menuDrawStatusMessage();
	oslEndDrawing();
}

void VideoGuScreenClear(void)
{
//	VideoGuTerm();
//	VideoGuInit();

	oslStartDrawing();
	oslClearScreen(RGB(0,0,0));
	oslEndDrawing();
	oslSwapBuffers();

	oslStartDrawing();
	oslClearScreen(RGB(0,0,0));
	oslEndDrawing();
	oslSwapBuffers();
}


#define SLICE_SIZE 64 // Change this to experiment with different page-cache sizes

void pspScaleImage(char *pixels, int srcX0, int srcY0, int srcX1, int srcY1, 
				   int srcPitch, int dstX0, int dstY0, int dstX1, int dstY1)
{
	int x, i, j;
	uint8 r, g, b;
	struct Vertex* vertices;
	float slice_scale = (SLICE_SIZE * (dstX1 - dstX0) / (srcX1 - srcX0));

	if (bitmap.depth == 8)		{
		if(bitmap.pal.update)
		{
			for(i=0; i<PALETTE_SIZE; i++)
			{
				r = bitmap.pal.color[i][0];
				g = bitmap.pal.color[i][1];
				b = bitmap.pal.color[i][2];

				Screen.pal[i] = MAKE_PIXEL(r,g,b);
				Screen.pal[i+PALETTE_SIZE] = MAKE_PIXEL(r,g,b);
				Screen.pal[i+(2*PALETTE_SIZE)] = MAKE_PIXEL(r,g,b);
				Screen.pal[i+(3*PALETTE_SIZE)] = MAKE_PIXEL(r,g,b);
			}
		}

		sceKernelDcacheWritebackInvalidateRange(Screen.pal, PALETTE_SIZE * 4 * 2);
		sceKernelDcacheWritebackInvalidateRange(pixels, 256 * 256);

		sceGuClutMode(GU_PSM_5551,0,0xff,0);
		sceGuClutLoad((256/8),Screen.pal);
		sceGuTexMode(GU_PSM_T8,0,0,0);
	}
	else
		sceGuTexMode(GU_PSM_5551,0,0,0);
	sceGuTexImage(0,256,256,srcPitch,pixels);
	sceGuDisable(GU_DITHER);
	oslSetBilinearFilter(menuConfig.video.smoothing);
	// do a striped blit (takes the page-cache into account)
	x=dstX0;

	for (j = srcX0; j < srcX1; j += SLICE_SIZE, x += slice_scale)
	{
		vertices = (struct Vertex*)sceGuGetMemory(2 * sizeof(struct Vertex));
		vertices[0].u = j; vertices[0].v = srcY0;
		vertices[0].x = x; vertices[0].y = dstY0; vertices[0].z = 0;
		vertices[1].u = min(j+SLICE_SIZE, srcX1); vertices[1].v = srcY1;
		vertices[1].x = min(x+slice_scale, dstX1); vertices[1].y = dstY1; 
		vertices[1].z = 0;

		sceGuDrawArray(GU_SPRITES,GU_TEXTURE_16BIT|GU_VERTEX_16BIT|GU_TRANSFORM_2D,2,0,vertices);
	}

	oslSetBilinearFilter(0);
	return;
}
