#include "pspcommon.h"

//Type: 1 = pointer (param1 = pointeur, param2 = taille), 2 = file (param1 = nom fichier)
VIRTUAL_FILE *VirtualFileOpen(void *param1, int param2, int type, const char *mode)			{
	VIRTUAL_FILE *f;
	f = (VIRTUAL_FILE*)malloc(sizeof(*f));
	if (f)		{
		memset(f, 0, sizeof(*f));
		if (type == VF_MEMORY)		{
			f->file = NULL;
			f->offset = 0;
			f->ptr = param1;
			f->maxSize = param2;
		}
		else if (type == VF_FILE)		{
			f->ptr = NULL;
			f->file = fopen((char*)param1, mode);
			//L'ouverture a chou
			if (!f->file)		{
				free(f);
				f = NULL;
			}
		}
	}
	return f;
}

void VirtualFileClose(VIRTUAL_FILE *f)		{
	if (f->file)
		fclose(f->file);
	free(f);
}

int VirtualFileWrite(const void *ptr, size_t size, size_t n, VIRTUAL_FILE* f)		{
	int realSize = size * n, writeSize;
	if (f->ptr)		{
		//Dbordement?
		writeSize = min(realSize, f->maxSize - f->offset);
		memcpy((char*)f->ptr + f->offset, ptr, writeSize);
		f->offset += realSize;
	}
	else if (f->file)
		writeSize = fwrite(ptr, size, n, f->file);
	return writeSize;
}

int VirtualFileRead(void *ptr, size_t size, size_t n, VIRTUAL_FILE* f)		{
	int readSize, realSize = size * n;
	if (f->ptr)		{
		//min => pour viter les dbordements
		readSize = min(realSize, f->maxSize - f->offset);
		memcpy(ptr, (char*)f->ptr + f->offset, readSize);
		f->offset += realSize;
	}
	else if (f->file)
		readSize = fread(ptr, size, n, f->file);
	return readSize;
}

int VirtualFileGetc(VIRTUAL_FILE *f)		{
	unsigned char car;
	//Pour la scurit, quand mme un cas  part pour les fichiers
	if (f->file)
		return fgetc(f->file);
	else	{
		if (VirtualFileRead(&car, sizeof(car), 1, f) < 1)
			return -1;
		else
			return (int)car;
	}
}

int VirtualFilePutc(int caractere, VIRTUAL_FILE *f)		{
	unsigned char car = caractere;
	if (f->file)
		fputc(caractere, f->file);
	else		{
		if (VirtualFileWrite(&car, sizeof(car), 1, f) < 1)
			return -1;
		else
			return caractere;
	}
}

void VirtualFilePuts(const char *s, VIRTUAL_FILE *f)		{
	VirtualFileWrite(s, strlen(s), 1, f);
}

int VirtualFileSeek(VIRTUAL_FILE *f, int offset, int whence)		{
	if (f->ptr)		{
		if (whence == SEEK_SET)
			f->offset = offset;
		else if (whence == SEEK_CUR)
			f->offset += offset;
		else if (whence == SEEK_END)
			f->offset = f->maxSize - offset;
		f->offset = max(min(f->offset, f->maxSize), 0);
	}
	else if (f->file)
		return fseek(f->file, offset, whence);
	return f->offset;
}

int VirtualFileEof(VIRTUAL_FILE *f)		{
	if (f->file)
		return feof(f->file);
	return (f->offset < f->maxSize);
}

char *VirtualFileGets(char *str, int maxLen, VIRTUAL_FILE *f)			{
	const int blockSize = 16;
	int offset = 0, i, size;
	while(1)			{
		size = VirtualFileRead(str + offset, 1, min(maxLen - offset, blockSize), f);
		if (offset + size < maxLen)
			str[offset + size] = 0;
		for (i=offset;i<offset+blockSize;i++)		{
			if (str[i] == 0)
				return str;
			//\r\n (Windows)
			if (str[i] == '\r')			{
				str[i] = 0;
				//Dernier bloc de la liste?
				if (i + 1 >= offset + blockSize)			{
					char temp[1];
					int tempSize;
					tempSize = VirtualFileRead(temp, 1, 1, f);
					//Prochain caractre est un \n?
					if (!(tempSize > 0 && temp[0] == '\n'))
						//Sinon on annule
						i--;
				}
				else	{
					if (str[i + 1] == '\n')
						i++;
				}
				//Retourne le pointeur
				VirtualFileSeek(f, -size + (i - offset) + 1, SEEK_CUR);
				return str;
			}
			else if (str[i] == '\n')			{
				str[i] = 0;
				//Retourne le pointeur
				VirtualFileSeek(f, -blockSize + i, SEEK_CUR);
				return str;
			}
		}
		offset += blockSize;
	}
	return str;
}
